<?php

namespace Drupal\batch\Batch\Operation;

use Drupal\awareness\Cache\MemoryCache\EntityMemoryCacheAwareTrait;
use Drupal\awareness\Entity\EntityTypeManagerAwareTrait;
use Drupal\Component\Utility\Bytes;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Base class for operations.
 */
abstract class OperationBase implements OperationInterface {

  use DependencySerializationTrait;
  use EntityMemoryCacheAwareTrait;
  use EntityTypeManagerAwareTrait;
  use LoggerChannelTrait;
  use MessengerTrait;
  use StringTranslationTrait;

  /**
   * The number of items to process during on processing run.
   *
   * @var int
   */
  protected int $itemsPerProcess = 10;

  /**
   * Set the number of items to process during each processing run.
   *
   * @param int $itemsPerProcess
   *   The number of items to process during each processing run.
   *
   * @return $this
   */
  public function setItemsPerProcess(int $itemsPerProcess): self {
    $this->itemsPerProcess = $itemsPerProcess;
    return $this;
  }

  /**
   * Reclaims memory.
   *
   * @return bool
   *   Indicates if memory was reclaimed.
   *
   * @see \Drupal\migrate\MigrateExecutable::attemptMemoryReclaim
   */
  protected function reclaimMemory(): bool {
    $limit = trim(ini_get('memory_limit'));
    if ($limit == '-1') {
      $limit = Bytes::toNumber('256MB');
    }
    else {
      $limit = Bytes::toNumber($limit);
    }
    if (memory_get_usage() / $limit < .85) {
      return FALSE;
    }

    drupal_static_reset();

    // Entity storage can blow up with caches so clear them out.
    foreach ($this->getEntityTypeManager()->getDefinitions() as $id => $definition) {
      $this->getEntityTypeManager()->getStorage($id)->resetCache();
    }

    // Clear the entity storage memory cache.
    /** @var \Drupal\Core\Cache\MemoryCache\MemoryCache $memory_cache */
    $this->getEntityMemoryCache()->deleteAll();

    // Run garbage collector to further reduce memory.
    gc_collect_cycles();
    return TRUE;
  }

  /**
   * Process an item.
   *
   * This is the main worker method for child classes.
   *
   * @param mixed $item
   *   An item to be processed.
   * @param array $context
   *   Batch context.
   */
  protected abstract function processItem(mixed $item, array &$context): void;

  /**
   * Initialize the batch.
   *
   * @param array $context
   *   Batch context.
   */
  protected function initializeBatch(array &$context): void {
    if (!empty($context['sandbox']['batch_init'])) {
      return;
    }
    $context['sandbox']['batch_init'] = TRUE;
    $this->init($context);
  }

  /**
   * Initialize the batch.
   *
   * @param array $context
   *   Batch context.
   */
  abstract protected function init(array &$context): void;

}
